/*
** Copyright 2017 Double Precision, Inc.
** See COPYING for distribution information.
*/
#ifndef hotspot_elementfwd_h
#define hotspot_elementfwd_h

#include "hotspot.H"
#include "x/w/elementobj.H"
#include "x/w/input_mask.H"
#include "x/w/button_eventfwd.H"
#include "x/w/key_eventfwd.H"
#include "x/w/focus.H"
#include "focus/focusable_elementfwd.H"
#include "shortcut/shortcut_activation_elementfwd.H"

LIBCXXW_NAMESPACE_START

//! A mixin for inherting from a display element, and a hotspotObj::implObj.

//! The template parameter must be a subclass of child_elementObj.
//! This mixin alos requires the focusable_element mixin. A hotspot is
//! always focusable, because it handles keyboard and pointer clicks.
//!
//! This template class multiply-inherits from the
//! focusable_elementObj mixin,  and hotspotObj::implObj.

template<typename element_impl=child_elementObj>
class LIBCXX_HIDDEN hotspot_elementObj
	: public shortcut_activation_elementObj<focusable_elementObj
						<element_impl>>,
	  public hotspotObj::implObj {

	typedef focusable_elementObj<element_impl> nonshortcut_superclass_t;

	typedef shortcut_activation_elementObj<nonshortcut_superclass_t
					       > superclass_t;

 public:

	using superclass_t::superclass_t;

	~hotspot_elementObj();

	child_elementObj &get_hotspot_element() override;

	const child_elementObj &get_hotspot_element() const override;

	focusableImplObj &get_hotspot_focusable() override;

	const focusableImplObj &get_hotspot_focusable() const override;

	void keyboard_focus(IN_THREAD_ONLY) override;

	void pointer_focus(IN_THREAD_ONLY) override;

	bool process_key_event(IN_THREAD_ONLY, const key_event &ke) override;
	bool process_button_event(IN_THREAD_ONLY,
				  const button_event &be,
				  xcb_timestamp_t timestamp) override;

	void install_shortcut(const shortcut &,
			      const activated_in_thread &) override;

	void uninstall_shortcut() override;
};


LIBCXXW_NAMESPACE_END

#endif
