/*
** Copyright 2017 Double Precision, Inc.
** See COPYING for distribution information.
*/

#ifndef x_w_rgb_H
#define x_w_rgb_H

#include <x/w/rgbfwd.H>
#include <x/w/namespace.H>
#include <stdlib.h>

LIBCXXW_NAMESPACE_START

//! RGB color, with an alpha channel

class rgb {

public:
	//! Red component
	rgb_component_t r;

	//! Green component
	rgb_component_t g;

	//! Blue component
	rgb_component_t b;

	//! Alpha component
	rgb_component_t a;

	//! Maximum intensity

	static constexpr rgb_component_t maximum=0xFFFF;

	//! Default constructor
	rgb() : r(0), g(0), b(0), a(maximum) {}

	//! Constructor
	rgb(rgb_component_t rv, rgb_component_t gv, rgb_component_t bv,
	    rgb_component_t av=maximum) : r(rv), g(gv), b(bv), a(av) {}

	//! Comparator

	bool operator==(const rgb &o) const
	{
		return r == o.r && g == o.g && b == o.b && a == o.a;
	}

	//! Comparator
	bool operator!=(const rgb &o) const
	{
		return !operator==(o);
	}

	//! Comparator, for associative containers

	bool operator<(const rgb &a) const
	{
		return value() < a.value();
	}

	//! Comparator, for associative containers

	uint64_t value() const noexcept
	{
		return (((((((uint64_t)a) << 16) | r) << 16) | g) << 16) | b;
	}
};

LIBCXXW_NAMESPACE_END

#endif
